package org.telosys.tools.generator;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.telosys.tools.commons.TelosysToolsLogger;
import org.telosys.tools.commons.Variable;
import org.telosys.tools.generator.config.IGeneratorConfig;
import org.telosys.tools.generator.context.AboutGenerator;
import org.telosys.tools.generator.context.EntityContext;
import org.telosys.tools.generator.context.JavaClass;
import org.telosys.tools.generator.context.ProjectConfiguration;
import org.telosys.tools.generator.context.Today;

/**
 * This class is a Velocity generator ready to use. <br>
 * It is not supposed to be used directly by the application ( visibility "package" ) <br>
 * It is designed to be used only by the GenerationManager <br>
 * 
 * It holds : <br>
 * . the template file to use <br>
 * . the Velocity Engine <br>
 * . the Velocity Context  <br>
 * <br> 
 * After creation, each instance of this class has a Velocity Context initialized with <br>
 * . the generator variables : $generator, $today <br>
 * . the project variables <br>
 * . 
 * 
 * @author Laurent Guerin
 *  
 */
public class Generator {

	//private final static boolean CREATE_DIR = true ;
	private final static boolean DO_NOT_CREATE_DIR = false ;
	
	private VelocityEngine _velocityEngine = null;

	private VelocityContext _velocityContext = null;

	private TelosysToolsLogger _logger = null;

	private String _sTemplateDirectory = null;

	private String _sTemplateFileName = null;


//	/**
//	 * Constructor 
//	 * 
//	 * @param sTemplateDirectory
//	 * @param sTemplateFileName
//	 * @param projectVariables
//	 * @param logger
//	 * @throws GeneratorException
//	 */
//	public Generator(String sTemplateDirectory, String sTemplateFileName, 
//			Variable[] projectVariables, TelosysToolsLogger logger) throws GeneratorException 
//	{
//		_logger = logger;
//		log("Generator constructor (" + sTemplateDirectory + ", "
//				+ sTemplateFileName + ")");
//
//		checkTemplate(sTemplateDirectory, sTemplateFileName);
//
//		_sTemplateDirectory = sTemplateDirectory;
//		_sTemplateFileName = sTemplateFileName;
//
//		//--- Create a context
//		log("Generator constructor : VelocityContext creation ...");
//		_velocityContext = new VelocityContext();
//		log("Generator constructor : VelocityContext created.");
//
//		log("Generator constructor : VelocityContext initialization ...");
//		initContext(projectVariables);
//		log("Generator constructor : VelocityContext initialized.");
//
//		log("Generator constructor : VelocityEngine initialization ...");
//		initEngine(_sTemplateDirectory);
//		log("Generator constructor : VelocityEngine initialized.");
//	}

	public Generator( String sTemplateFileName, 
			IGeneratorConfig generatorConfig, TelosysToolsLogger logger) throws GeneratorException 
	{
		_logger = logger;
		log("Generator constructor (" + sTemplateFileName + ")");

//		checkTemplate(sTemplateDirectory, sTemplateFileName);
//		_sTemplateDirectory = sTemplateDirectory;
//		_sTemplateFileName = sTemplateFileName;

		if ( null == sTemplateFileName) {
			throw new GeneratorException("Template file name is null (Generator constructor argument)");
		}
		if ( null == generatorConfig) {
			throw new GeneratorException("Generator configuration is null (Generator constructor argument)");
		}
		
		//------------------------------------------------------------------
		// 1) Init context
		//------------------------------------------------------------------
		//--- Create a context
		log("Generator constructor : VelocityContext creation ...");
		_velocityContext = new VelocityContext();
		log("Generator constructor : VelocityContext created.");

		//--- Get the project configuration instance for "$project"
		ProjectConfiguration projectConfiguration = generatorConfig.getProjectConfiguration();

		log("Generator constructor : VelocityContext initialization ...");
//		initContext(projectConfiguration, projectVariables);
		initContext(projectConfiguration);
		log("Generator constructor : VelocityContext initialized.");
		
		
		//------------------------------------------------------------------
		// 2) Init engine
		//------------------------------------------------------------------
		//--- Get the templates directory and use it to initialize the engine		
		String sTemplateDirectory = generatorConfig.getTemplatesFolderFullPath();		
		log("Templates Directory : '" + sTemplateDirectory + "'");

		//--- Check template file existence		
		checkTemplate(sTemplateDirectory, sTemplateFileName);
		_sTemplateDirectory = sTemplateDirectory;
		_sTemplateFileName  = sTemplateFileName;

		log("Generator constructor : VelocityEngine initialization ...");
		initEngine(_sTemplateDirectory);
		log("Generator constructor : VelocityEngine initialized.");
	}

	private void log(String s) {
		if (_logger != null) {
			_logger.log(s);
		}
	}

	private void checkTemplate(String sTemplateDirectory,
			String sTemplateFileName) throws GeneratorException {
		if (sTemplateDirectory == null) {
			throw new GeneratorException("Template directory is null !");
		}
		if (sTemplateFileName == null) {
			throw new GeneratorException("Template file name is null !");
		}
		File dir = new File(sTemplateDirectory);
		if (!dir.exists()) {
			throw new GeneratorException("Template directory '"
					+ sTemplateDirectory + "' doesn't exist !");
		}
		if (!dir.isDirectory()) {
			throw new GeneratorException("Template directory '"
					+ sTemplateDirectory + "' is not a directory !");
		}

		String sTemplateFullPath = null;
		if (sTemplateDirectory.endsWith("/")) {
			sTemplateFullPath = sTemplateDirectory + sTemplateFileName;
		} else {
			sTemplateFullPath = sTemplateDirectory + "/" + sTemplateFileName;
		}
		File file = new File(sTemplateFullPath);
		if (!file.exists()) {
			throw new GeneratorException("Template file '" + sTemplateFullPath
					+ "' doesn't exist !");
		}
		if (!file.isFile()) {
			throw new GeneratorException("Template file '" + sTemplateFullPath
					+ "' is not a file !");
		}
	}
	private void checkTargetFile(String sFile, boolean bCreateDir) throws GeneratorException {
		if (sFile == null) {
			throw new GeneratorException("Target file is null !");
		}
		File file = new File(sFile);
		if ( file.exists() )
		{
			if ( ! file.canWrite() )				
			{
				throw new GeneratorException("Cannot write on target file '"+ file.toString() + "' !");
			}
		}
		else
		{
			File parent = file.getParentFile();
			if ( ! parent.exists() )
			{
				if ( bCreateDir == false )
				{
					throw new GeneratorException("Target directory '"+ parent.toString() + "' not found !");
				}
				else
				{
					// Create the target file directory(ies)
					parent.mkdirs();				
				}
			}
		}
	}
	//========================================================================
	// CONTEXT MANAGEMENT
	//========================================================================
	//private void initContext( ProjectConfiguration projectConfiguration, Variable[] projectVariables ) 
	private void initContext( ProjectConfiguration projectConfiguration ) 
	{
		log("initContext()..." );
		
		//--- Set the standard Velocity variables in the context
		_velocityContext.put("generator", new AboutGenerator());
		_velocityContext.put("today", new Today());
		_velocityContext.put("class", null);
		
		//--- Set the "$project" variable in the context
		_velocityContext.put("project", projectConfiguration);
		
		//--- Get the project variables and put them in the context	
		Variable[] projectVariables = projectConfiguration.getVariables();
		log("initContext() : Project variables count = " + ( projectVariables != null ? projectVariables.length : 0 ) );

		//--- Set the project variables in the context ( if any )
		if ( projectVariables != null )
		{
			for ( int i = 0 ; i < projectVariables.length ; i++ )
			{
				Variable var = projectVariables[i];
				_velocityContext.put( var.getName(), var.getValue() );
			}
		}
	}

//	public final static String BEAN_CLASS_CONTEXT_NAME       = "beanClass" ;
//	public final static String LIST_CLASS_CONTEXT_NAME       = "listClass" ;
//	public final static String DAO_CLASS_CONTEXT_NAME        = "daoClass" ;
//	public final static String CONVERTER_CLASS_CONTEXT_NAME  = "xmlClass" ;
//
//	public final static String CURRENT_CLASS_CONTEXT_NAME    = "class" ;
//
//	public final static int JAVA_CLASS_TYPE_UNDEFINED = 0 ;
//	public final static int JAVA_CLASS_TYPE_BEAN      = 1 ;
//	public final static int JAVA_CLASS_TYPE_LIST      = 2 ;
//	public final static int JAVA_CLASS_TYPE_DAO       = 3 ;
//	public final static int JAVA_CLASS_TYPE_CONVERTER = 4 ;
	
	public void initEntityContext( EntityContext entityContext, int javaClassTargetType ) 
	{
		_velocityContext.put(GeneratorConst.BEAN_CLASS_CONTEXT_NAME,      entityContext.getBeanClass() );
		_velocityContext.put(GeneratorConst.LIST_CLASS_CONTEXT_NAME,      entityContext.getListClass() );
		_velocityContext.put(GeneratorConst.DAO_CLASS_CONTEXT_NAME,       entityContext.getDaoClass() );
		_velocityContext.put(GeneratorConst.CONVERTER_CLASS_CONTEXT_NAME, entityContext.getConverterClass() );
		
//		defineTargetType ( javaClassTargetType );
		//--- Set the "$class" variable in the context if the target is a predefined Java Class target
		switch ( javaClassTargetType )
		{
		case GeneratorConst.JAVA_CLASS_TYPE_BEAN :
			setInContextIfNotNull( GeneratorConst.CURRENT_CLASS_CONTEXT_NAME, entityContext.getBeanClass() );
			break;
		case GeneratorConst.JAVA_CLASS_TYPE_LIST :
			setInContextIfNotNull( GeneratorConst.CURRENT_CLASS_CONTEXT_NAME, entityContext.getListClass() );
			break;
		case GeneratorConst.JAVA_CLASS_TYPE_DAO :
			setInContextIfNotNull( GeneratorConst.CURRENT_CLASS_CONTEXT_NAME, entityContext.getDaoClass() );
			break;
		case GeneratorConst.JAVA_CLASS_TYPE_CONVERTER :
			setInContextIfNotNull( GeneratorConst.CURRENT_CLASS_CONTEXT_NAME, entityContext.getConverterClass() );
			break;
//		default :
//			throw new GeneratorException("Cannot define target type (invalid type)");
		}
	}
	
//	/**
//	 * Set the "$class" variable in the context according with the given target type
//	 * @param targetType
//	 * @throws GeneratorException
//	 */
//	private void defineTargetType ( int targetType ) //throws GeneratorException
//	{
//		switch ( targetType )
//		{
//		case JAVA_CLASS_TYPE_BEAN :
//			setInContextIfNotNull( CURRENT_CLASS_CONTEXT_NAME, _velocityContext.get(BEAN_CLASS_CONTEXT_NAME));
//			break;
//		case JAVA_CLASS_TYPE_LIST :
//			setInContextIfNotNull( CURRENT_CLASS_CONTEXT_NAME, _velocityContext.get(LIST_CLASS_CONTEXT_NAME));
//			break;
//		case JAVA_CLASS_TYPE_DAO :
//			setInContextIfNotNull( CURRENT_CLASS_CONTEXT_NAME, _velocityContext.get(DAO_CLASS_CONTEXT_NAME));
//			break;
//		case JAVA_CLASS_TYPE_CONVERTER :
//			setInContextIfNotNull( CURRENT_CLASS_CONTEXT_NAME, _velocityContext.get(CONVERTER_CLASS_CONTEXT_NAME));
//			break;
////		default :
////			throw new GeneratorException("Cannot define target type (invalid type)");
//		}
//	}
	
	private void setInContextIfNotNull ( String name, Object o )
	{
		if ( name != null && o != null )
		{
			_velocityContext.put(name, o);
		}
	}
	
	/**
	 * Set the current JavaClass target in the context ( the "$class" variable )
	 * @param javaClass
	 */
	public void setJavaClassTargetInContext(JavaClass javaClass) 
	{
		//_velocityContext.put("class", javaClass);
		_velocityContext.put(GeneratorConst.CURRENT_CLASS_CONTEXT_NAME, javaClass);
	}
	
	/**
	 * Returns the JavaClass ( the "$class" variable ) from the context (or null if not set)
	 * @return
	 */
	public JavaClass getJavaClassTargetFromContext() 
	{
		//Object o = _velocityContext.get("class");
		Object o = _velocityContext.get(GeneratorConst.CURRENT_CLASS_CONTEXT_NAME);
		if ( o != null )
		{
			if ( o instanceof JavaClass )
			{
				return (JavaClass) o ;
			}
		}
		return null ;
	}

	/**
	 * Set a new attribute (variable) in the Velocity Context
	 * @param sName
	 * @param oValue
	 */
	public void setContextAttribute(String sName, Object oValue) 
	{
		_velocityContext.put(sName, oValue);
	}

	/**
	 * Initialize the Velocity engine Each instance of "Generator" has its own
	 * Velocity engine
	 * 
	 * @return
	 */
	private int initEngine(String sTemplateDirectory) throws GeneratorException 
	{
		log("   ... Init Velocity engine");

		_velocityEngine = new VelocityEngine();
		_velocityEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH, sTemplateDirectory);

		try {
			_velocityEngine.init();
		} catch (Exception e) {
			throw new GeneratorException("Cannot init VelocityEngine", e );
		}

		return 0;
	}

	/**
	 * Return a PrintWriter for a specific type
	 * 
	 * @param iType
	 * @return
	 */
	private PrintWriter getPrintWriter(String sFileName) {
		//        File dir = null;
		//        log(" -- GetPrintWriter(" + sFileName + ")");
		FileWriter fw = null;
		//
		//        //--- Instance of Directory object + creation on the filesystem
		//        //sDirectory = sDirectory.replace('.', '/');
		//        log(" -- Directory = " + _sDestDirectory);
		//        dir = new File(_sDestDirectory);
		//        if (!dir.exists())
		//        {
		//            log(" -- Create directory " + _sDestDirectory);
		//            if (dir.mkdirs())
		//            {
		//                log(" -- Directory created.");
		//            }
		//            else
		//            {
		//                log(" ### Error : " + _sDestDirectory + " is not created !");
		//                return null;
		//            }
		//        }
		//        else
		//        {
		//            if (!dir.isDirectory())
		//            {
		//                log(" ### Error : " + _sDestDirectory + " is not a directory
		// !");
		//                return null;
		//            }
		//        }

		//--- Full file path
		//        String sFullPath = _sDestDirectory + "/" + sFileName;
		String sFullPath = sFileName;
		log("      -- Full Path = " + sFullPath);
		try {
			fw = new FileWriter(sFullPath);
		} catch (IOException e) {
			log("   ### error in getPrintWriter : cannot create FileWriter for "
							+ sFullPath);
			return null;
		}
		PrintWriter pw = new PrintWriter(fw, false);
		//log("End of getPrintWriter().");
		return pw;
	}

	private Template getTemplate() throws GeneratorException {
		if (_velocityEngine == null) {
			throw new GeneratorException("Velocity engine is null!");
		}
		log("getTemplate() : Template file name = '" + _sTemplateFileName + "'");
		Template template = null;
		try {
			template = _velocityEngine.getTemplate(_sTemplateFileName);
		} catch (ResourceNotFoundException e) {
			e.printStackTrace();
			throw new GeneratorException("Cannot get template : ResourceNotFoundException ! ", e );
		} catch (ParseErrorException e) {
			e.printStackTrace();
			throw new GeneratorException("Cannot get template : Velocity ParseErrorException ! ", e );
		} catch (Exception e) {
			e.printStackTrace();
			throw new GeneratorException("Cannot get template : Exception ! ", e );
		}
		return template;
	}

	private void generate(Writer writer, Template template)
			throws GeneratorException {
		log("generate(writer, template)...");
		//_contextHolder.initJavaClass(iType);
		try {
			//--- Generate in a Writer
			template.merge(_velocityContext, writer);
		} catch (ResourceNotFoundException e) {
			throw new GeneratorException(
					"generate(w,t) : ResourceNotFoundException ", e);
		} catch (ParseErrorException e) {
			throw new GeneratorException(
					"generate(w,t) : ParseErrorException ", e);
		} catch (MethodInvocationException e) {
			throw new GeneratorException(
					"generate(w,t) : MethodInvocationException ", e);
		} catch (Exception e) {
			throw new GeneratorException("generate(w,t) : Exception ", e);
		}
	}

	public void generate(Writer writer) throws GeneratorException {
		log("generate(writer) : getTemplate() ...");
		Template template = getTemplate();
		log("generate(writer) : generate(writer, template) ...");		
		generate(writer, template);
	}

	public InputStream generateStream() throws GeneratorException {
		log("generateStream()...");
		StringWriter stringWriter = new StringWriter();
		generate(stringWriter);
		byte[] bytes = stringWriter.toString().getBytes();
		return new ByteArrayInputStream(bytes);
	}

	public void generateFile(String sFileName) throws GeneratorException {
		generateFile(sFileName, DO_NOT_CREATE_DIR); 
	}
	
	public void generateFile(String sFileName, boolean bCreateDir ) throws GeneratorException {
		log("generateFile(" + sFileName + ")...");
		checkTargetFile(sFileName, bCreateDir);
		StringWriter stringWriter = new StringWriter();
		generate(stringWriter);

		//--- File creation
		PrintWriter pw = this.getPrintWriter(sFileName);
		if (pw == null) {
			throw new GeneratorException(
					"generateFile : cannot get PrintWriter ");
		}
		pw.println(stringWriter.toString());
		pw.flush();
		pw.close();
	}

}