package org.telosys.tools.generator.config;

import java.util.Properties;

import org.telosys.tools.commons.Variable;
import org.telosys.tools.commons.VariablesUtil;
import org.telosys.tools.generator.context.ProjectConfiguration;

public class GeneratorConfig implements IGeneratorConfig
{
	public static final String PROJECT_CONFIG_FILE = "telosys-tools.cfg" ;
	
	private String _sProjectLocation ; // Project folder ( absolute path )
	
	private Variable[] _projectVariables ;
	
	private String _sSourceFolder ;
	private String _sWebContentFolder ;
	
	private String _sTemplatesFolder ;
	//private String _sRepositoriesFolder ;
	
	private String _sPackageVO ;
	private String _sPackageVOList ;
	private String _sPackageXmlMapper ;
	private String _sPackageDAO ;
	
	private String _sPackageScreenData ;
	private String _sPackageScreenManager ;
	private String _sPackageScreenProcedures ;
	private String _sPackageScreenTriggers ;

	public GeneratorConfig(String sProjectLocation, Properties prop) 
	{
		_sProjectLocation = sProjectLocation;
		
		//--- Files folders
    	_sSourceFolder       = prop.getProperty(GeneratorConfigConst.SOURCE_FOLDER, _sSourceFolder);
    	_sWebContentFolder   = prop.getProperty(GeneratorConfigConst.WEB_CONTENT_FOLDER, _sWebContentFolder);
    	_sTemplatesFolder    = prop.getProperty(GeneratorConfigConst.TEMPLATES_FOLDER, _sTemplatesFolder);
    	//_sRepositoriesFolder = prop.getProperty(GeneratorConfigConst.REPOS_FOLDER, _sRepositoriesFolder);
		
		//--- Packages names
    	_sPackageVO        = prop.getProperty(GeneratorConfigConst.PACKAGE_VO, _sPackageVO);
    	_sPackageVOList    = prop.getProperty(GeneratorConfigConst.PACKAGE_VO_LIST, _sPackageVOList);
    	_sPackageXmlMapper = prop.getProperty(GeneratorConfigConst.PACKAGE_XML_MAPPER, _sPackageXmlMapper);
    	_sPackageDAO       = prop.getProperty(GeneratorConfigConst.PACKAGE_DAO, _sPackageDAO);
    	
    	_sPackageScreenData       = prop.getProperty(GeneratorConfigConst.PACKAGE_SCREEN_DATA, _sPackageScreenData);
    	_sPackageScreenManager    = prop.getProperty(GeneratorConfigConst.PACKAGE_SCREEN_MANAGER, _sPackageScreenManager);
    	_sPackageScreenProcedures = prop.getProperty(GeneratorConfigConst.PACKAGE_SCREEN_PROCEDURES, _sPackageScreenProcedures);
    	_sPackageScreenTriggers   = prop.getProperty(GeneratorConfigConst.PACKAGE_SCREEN_TRIGGERS, _sPackageScreenTriggers);
    	
    	//--- Project user defined variables
    	_projectVariables = VariablesUtil.getVariablesFromProperties( prop );
    	
	}

	//---------------------------------------------------------------------
	// Specific variables
	//---------------------------------------------------------------------
	public Variable[] getProjectVariables() 
	{
		return _projectVariables ;
	}

	//---------------------------------------------------------------------
	// Project Configuration for Generator context
	//---------------------------------------------------------------------
	public ProjectConfiguration getProjectConfiguration() 
	{
		ProjectConfiguration projectConfiguration = new ProjectConfiguration( 
				_sSourceFolder, _sWebContentFolder,
				_sPackageVO, _sPackageVOList, _sPackageDAO, _sPackageXmlMapper,
				_sPackageScreenData, _sPackageScreenManager, _sPackageScreenProcedures, _sPackageScreenTriggers,
				_projectVariables );
		
		return projectConfiguration;
	}

	//---------------------------------------------------------------------
	// Folders
	//---------------------------------------------------------------------
    public String getProjectLocation()
	{
    	return _sProjectLocation ;
	}
    
    /**
     * Returns the "Project Full Path" by adding the given subpath at the end of the project location
     * @param sSubPath the "sub path" to add
     * @return
     */
    private String getProjectFullPath(String sSubPath)
	{
    	if ( sSubPath != null )
    	{
    		String s = sSubPath.trim() ;
    		if ( s.startsWith("/") || s.startsWith("\\") )
    		{
    	    	return _sProjectLocation + s ;
    		}
    		else
    		{
    	    	return _sProjectLocation + "/" + s ;
    		}
    	}
    	else
    	{
    		return _sProjectLocation ;
    	}
	}
    
	
    /**
     * Returns the "full path" templates folder (never null)<br>
     * e.g. : "C:/dir/workspace/project/folder"
     * It can be the plugin templates folder ( if the template folder property is not set )
     * or the project templates folder
     * @return
     */
    public String getTemplatesFolderFullPath()
	{
    	if ( _sTemplatesFolder != null )
    	{
    		String s = _sTemplatesFolder.trim() ;
    		if ( s.length() == 0 )
    		{
    			// No templates folder 
    			return null ;
    		}
    		else
    		{
    			// Templates folder is set => build the full path
    			return getProjectFullPath(_sTemplatesFolder) ;
    		}
    	}
    	else
    	{
    		return null ;
    	}
	}

	//---------------------------------------------------------------------
	// Packages
	//---------------------------------------------------------------------
	public String getVOPackage() {
		return _sPackageVO ;
	}

	public String getVOListPackage() {
		return _sPackageVOList ;
	}

	public String getDaoPackage() {
		return _sPackageDAO ;
	}

	public String getXmlMapperPackage() {
		return _sPackageXmlMapper ;
	}


	public String getScreenDataPackage() {
		return _sPackageScreenData ;
	}

	public String getScreenManagerPackage() {
		return _sPackageScreenManager ;
	}
	
	public String getScreenProceduresPackage() {
		return _sPackageScreenProcedures ;
	}
	public String getScreenTriggersPackage() {
		return _sPackageScreenTriggers ;
	}
	
}
