package org.telosys.tools.generator.context;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.telosys.tools.commons.JavaClassUtil;
import org.telosys.tools.commons.JavaTypeUtil;

/**
 * List of full java class names that need imports<br>
 * eg : "java.util.Date", "java.math.BigDecimal", etc ...
 * 
 * @author Laurent GUERIN
 *
 */
public class JavaBeanClassImports {

	//private final TelosysToolsLogger _logger;
	
	private LinkedList<String> _list = new LinkedList<String>() ; // List of Java "full types" to import ( eg : "java.math.BigDecimal" )
	
	//public JavaBeanClassImports(final TelosysToolsLogger logger) 
	public JavaBeanClassImports() 
	{
		super();
		//this._logger = logger;
	}

	public List<String> getList()
	{
		return _list ;
	}
	
	/**
	 * Returns true if the given type is already declared ( stored in the list )
	 * @param type
	 * @return
	 */
	public boolean isDeclared(String type)
	{
		if ( type != null )
		{
			Iterator iter = _list.iterator();
			while ( iter.hasNext() )
			{
				String s2 = (String) iter.next();
				if ( s2.equals(type))
				{
					return true ; // Found = declared
				}
			}
		}
		return false ;
	}
	
	/**
	 * Declare the given type : stored only if needed ( requires import and not yet stored )
	 * @param type 
	 */
	public void declareType(String type )
	{
		if ( JavaTypeUtil.needsImport(type) != true )
		{
			return ;
		}
		if ( isDeclared(type) )
		{
			return ;
		}
		_list.add(type);
	}
	
	private int shortNameCount(String shortName)
	{
		//this._logger.log("shortNameCount(" + shortName + ") ... " );
		int count = 0 ;
		String end = "." + shortName ;
		Iterator iter = _list.iterator();
		while ( iter.hasNext() )
		{
			String s = (String) iter.next();
			if ( s.endsWith(end))
			{
				count++ ;
			}
		}
		//this._logger.log("shortNameCount(" + shortName + ") : return " + count );
		return count ;
	}
	
	private void registerCollidedTypes(String shortName, LinkedList<String> listExtracted )
	{
		String end = "." + shortName ;
		//--- store the types to extract 
		Iterator iter = _list.iterator();
		while ( iter.hasNext() )
		{
			String sFullType = (String) iter.next();
			if ( sFullType.endsWith(end))
			{
				listExtracted.add(sFullType);
			}
		}
	}
	
//	public void extract(LinkedList listExtracted )
//	{
//		Iterator iter = listExtracted.iterator();
//		while ( iter.hasNext() )
//		{
//			Object o = iter.next();
//			_list.remove(o); // the reference is the same 
//		}
//	}
	
	/**
	 * Extract the duplicated short names from the imports list to avoid imports collision
	 * eg : "import java.util.Date;" collides with "import java.sql.Date;"
	 * @return list of all the collided full types (those types are already extracted from imports),<br>
	 * or null if nothing has been extracted
	 */
	public LinkedList extractDuplicatedShortNames()
	{
		//this._logger.log("extractDuplicatedShortNames ... " );
		LinkedList<String> collidedTypes = null ;

		//--- Build the list of types to extract
		Iterator iter = _list.iterator();
		while ( iter.hasNext() )
		{
			String s = (String) iter.next();
			String shortName = JavaClassUtil.shortName(s);
			
			// if more than one occurrence of this short name in the imports 
			// ( eg  2 occurrences : "java.util.Date" and "java.sql.Date" for the "Date" short name)
			if ( shortNameCount(shortName) > 1 ) 
			{
				if ( collidedTypes == null )
				{
					collidedTypes = new LinkedList<String>() ;
				}
				registerCollidedTypes(shortName, collidedTypes );
			}
		}
		
		//--- Remove all the types to extract (if any)
		if ( collidedTypes != null )
		{
			_list.removeAll(collidedTypes);
		}
		
		//--- Return the list of types extracted
		return collidedTypes ;
	}
}
