package org.telosys.tools.generator.context.tools;

import org.telosys.tools.commons.JavaTypeUtil;
import org.telosys.tools.commons.StrUtil;
import org.telosys.tools.generator.context.JavaBeanClassAttribute;

public class AnnotationsForBeanValidation //extends AnnotationsBuilder
{
	private JavaBeanClassAttribute _attribute = null ;
	

//	private StringBuffer _sbAnnotationsBuffer = null;
//
//	private String       _sLeftMargin = "" ;
//	
//	private int          _iAnnotationsCount = 0 ;
//	
	public AnnotationsForBeanValidation(JavaBeanClassAttribute attribute) {
		super();
		this._attribute = attribute;
	}

	public String getValidationAnnotations( int iLeftMargin, String sFlagJSR303 )
	{
		boolean bJSR303strict = false ;
		if ( sFlagJSR303 != null ) {
			if ( sFlagJSR303.equalsIgnoreCase("JSR303") ) {
				bJSR303strict = true ;
			}
		}
		
		//--- Reset everything at each call 
		//resetAnnotations(iLeftMargin);
		AnnotationsBuilder annotations = new AnnotationsBuilder(iLeftMargin);

//		_sbAnnotationsBuffer = new StringBuffer();
//		_iAnnotationsCount = 0 ;
//		_sLeftMargin = GeneratorUtil.blanks(iLeftMargin);
		
		String sJavaFullType = _attribute.getFullType() ;
		
		//--- Annotations for all categories 
		if ( ! JavaTypeUtil.isPrimitiveType(sJavaFullType) ) 
		{
			if ( _attribute.isNotNull() ) 
			{
				annotations.addLine("@NotNull" );
			}
		}

		//--- Annotations for each type category 
		if ( JavaTypeUtil.isCategoryBoolean( sJavaFullType ) )
		{
			// Nothing to do !
		}
		else if ( JavaTypeUtil.isCategoryString( sJavaFullType ) )
		{
			annotationSize(annotations);
			annotationPattern(annotations);
			if ( ! bJSR303strict )
			{
				if ( _attribute.isNotEmpty() ) 
				{
					annotations.addLine("@NotEmpty" );
				}
				if ( _attribute.isNotBlank() ) 
				{
					annotations.addLine("@NotBlank" );
				}
			}
			
		}
		else if ( JavaTypeUtil.isCategoryNumber( sJavaFullType ) )
		{
			annotationMin(annotations);
			annotationMax(annotations);
		}
		else if ( JavaTypeUtil.isCategoryDateOrTime( sJavaFullType ) )
		{
			if ( _attribute.isDatePast() ) 
			{
				annotations.addLine("@Past" );
			}
			if ( _attribute.isDateFuture() ) 
			{
				annotations.addLine("@Future" );
			}
		}
		
		//return _sbAnnotationsBuffer.toString() ;
		return annotations.getAnnotations() ;
	}
	
	private void annotationSize(AnnotationsBuilder annotations)
	{
		String min = null ;
		String max = null ;
		if ( ! StrUtil.nullOrVoid ( _attribute.getMaxLength() ) )
		{
			max = _attribute.getMaxLength().trim() ;
			if ( ! StrUtil.nullOrVoid ( _attribute.getMinLength() ) )
			{
				min = _attribute.getMinLength().trim() ;
				annotations.addLine("@Size( min = " + min + ", max = " + max + " )");
			}
			else
			{
				annotations.addLine("@Size( max = " + max + " )");
			}
		}
		else
		{
			if ( ! StrUtil.nullOrVoid ( _attribute.getMinLength() ) )
			{
				min = _attribute.getMinLength().trim() ;
				annotations.addLine("@Size( min = " + min + " )");
			}
		}
	}


	private void annotationPattern(AnnotationsBuilder annotations)
	{
		if ( ! StrUtil.nullOrVoid ( _attribute.getPattern() ) )
		{
			annotations.addLine("@Pattern( regexp = \"" + _attribute.getPattern() + "\" )");
		}
	}
	
	private void annotationMin(AnnotationsBuilder annotations)
	{
		if ( ! StrUtil.nullOrVoid ( _attribute.getMinValue() ) )
		{
			String min = _attribute.getMinValue().trim() ;
			if ( min.indexOf('.') >=0 ) {
				annotations.addLine("@DecimalMin( value = \"" + min + "\" )");
			}
			else {
				annotations.addLine("@Min( value=" + min + " )");
			}
		}
	}
	private void annotationMax(AnnotationsBuilder annotations)
	{
		if ( ! StrUtil.nullOrVoid ( _attribute.getMaxValue() ) )
		{
			String max = _attribute.getMaxValue().trim() ;
			if ( max.indexOf('.') >=0 ) {
				annotations.addLine("@DecimalMax( value = \"" + max + "\" )");
			}
			else {
				annotations.addLine("@Max( value=" + max + " )");
			}
		}
	}
	
//	private void addAnnotation(String sAnnotation)
//	{
//		if ( _iAnnotationsCount > 0 ) {
//			_sbAnnotationsBuffer.append( "\n" );
//		}
//		_sbAnnotationsBuffer.append( _sLeftMargin );
//		_sbAnnotationsBuffer.append( sAnnotation );
//		_iAnnotationsCount++;
//	}

}
