package org.telosys.tools.generator.target;


/**
 * A generation target definition : what file to generate, where, with which template, etc...
 * 
 * @author Laurent Guerin
 *
 */
public class GenericTarget 
{
	//--- Special values for predefined Java Class targets
	public final static String JAVA_CLASS_FOR_BEAN = "#bean" ;
	public final static String JAVA_CLASS_FOR_LIST = "#list" ;
	public final static String JAVA_CLASS_FOR_DAO  = "#dao"  ;
	public final static String JAVA_CLASS_FOR_CONV = "#conv" ;
	

	private String  _sName = null ;

	private String  _sFile = null ;
	
	private String  _sFolder = null ;

	private String  _sTemplate = null ;

	/**
	 * @param name the target name (to be displayed in the UI)
	 * @param file the file to be generated ( ie "${BEANNAME}Data.java" )
	 * @param folder the folder where to generate the file ( ie "src/org/demo/screen/${BEANNAME_LC}" )
	 * @param template the template to use ( ie "vo_screen_data.vm" )
	 */
	public GenericTarget(String name, String file, String folder, String template) 
	{
		super();
		_sName = name;
		_sFile = file;
		_sFolder = folder;
		_sTemplate = template;
	}
	
	/**
	 * Copy constructor
	 * @param t
	 */
	public GenericTarget( GenericTarget t) 
	{
		super();
		_sName = t.getName();
		_sFile = t.getFile();
		_sFolder = t.getFolder();
		_sTemplate = t.getTemplate();
	}
	
	//-----------------------------------------------------------------------
	/**
	 * Returns the target name ( the text displayed on the screen )
	 * @return
	 */
	public String getName()
	{
		return _sName ;
	}
	
	/**
	 * Returns the target file name ( file to be generated )
	 * Can contains a generic variable BEANNAME, BEANNAME_UC, BEANNAME_LC
	 * if the target if generic and "applyBeanClassName" as not been called
	 * @return
	 */
	public String getFile()
	{
		return _sFile ;
	}
	
	/**
	 * Returns the folder where to generate the file
	 * Can contains a generic variable BEANNAME, BEANNAME_UC, BEANNAME_LC
	 * if the target if generic and "applyBeanClassName" as not been called
	 * @return
	 */
	public String getFolder()
	{
		return _sFolder ;
	}
	
	public String getFullFileName()
	{
		if ( _sFolder.endsWith("/") || _sFolder.endsWith("\\") )
		{
			return _sFolder + _sFile ;
		}
		return _sFolder + "/" + _sFile ;
	}
	
	/**
	 * Returns the template 
	 * @return
	 */
	public String getTemplate()
	{
		return _sTemplate ;
	}	
	
	public String toString()
	{
		return _sName + " : '" + _sFile  + "' in '" + _sFolder + "' ( " + _sTemplate + " )" ;
	}
}
